#ifndef GLOBALSETTINGSPANEL_H
#define GLOBALSETTINGSPANEL_H

#include "../../gui/settingspanel.h"

using namespace std;

class Globalsettingspanel : public Settingspanel
{
	public:
		Globalsettingspanel();
		~Globalsettingspanel();

	private:
};

#endif
