#ifndef ARCHMOUTH_H
#define ARCHMOUTH_H

using namespace std;

class archMouth
{
	public:
		archMouth();
		~archMouth();

		unsigned int id;

		// dimension
		float x;
		float y;
		float z;

	private:
};

#endif
