
#include "glscene.h"

GLScene::GLScene()
{
}

void GLScene::init()
{
}

void GLScene::draw()
{
}

void GLScene::handlekeyPressed(const SDLKey& key)
{
}

void GLScene::handlekeyReleased(const SDLKey& key)
{
}

void GLScene::handlemousebuttonPressed(int x, int y, const int&)
{
}

void GLScene::handlemousebuttonReleased(int x, int y, const int&)
{
}

void GLScene::handleMouseMotionRel(int x, int y)
{
}

void GLScene::handleMouseMotionAbs(int x, int y)
{
}

void GLScene::clean()
{
}

GLScene::~GLScene()
{
}
