#ifndef SETTINGSBRAINPANEL_H
#define SETTINGSBRAINPANEL_H

#include "../../gui/settingspanel.h"

using namespace std;

class Settingsbrainpanel : public Settingspanel
{
	public:
		Settingsbrainpanel();
		~Settingsbrainpanel();

	private:
};

#endif
